/*
 * img.c
 * Dan Wallach <dwallach@cs.princeton.edu>
 * 26 September 1995
 *
 * Stuff to convert from Tk images to a more sane data structure and back
 *
 * $Id: img.c,v 1.2 1995/09/28 03:43:50 dwallach Exp $
 */
 
#include <malloc.h>
#include "img.h"

int clamp(double x) {
  if ((x)<0) return 0; if ((x)>255) return 255; return (int)(x);
}

/*
 * constructors and destructors
 */
Img*
Img_New(char* szTkPhotoName) {
    Img* piNew = (Img *)malloc(sizeof(Img));
    if(piNew == NULL) return NULL;
    
    piNew->width = 0;
    piNew->height = 0;
    piNew->pix = NULL;
    
    if(Img_GetTk(piNew, szTkPhotoName) == -1) {
        free(piNew);
	return NULL;
    }
    
    return piNew;
}

Img*
Img_NewBlank(int iWidth, int iHeight) {
    int i;
    Img* piNew;
    
    piNew = (Img *)malloc(sizeof(Img));
    if(piNew == NULL) return NULL;
    
    piNew->width = iWidth;
    piNew->height = iHeight;
    piNew->pix = (ImgPixel *)calloc(sizeof(ImgPixel), iWidth*iHeight);
    if(piNew->pix == NULL) {
        free(piNew);
	return NULL;
    }
    
    return piNew;
}


FloatImg*
FloatImg_NewBlank(int iWidth, int iHeight) {
    int i;
    FloatImg* piNew;
    
    piNew = (FloatImg *)malloc(sizeof(FloatImg));
    if(piNew == NULL) return NULL;
    
    piNew->width = iWidth;
    piNew->height = iHeight;
    piNew->pix = (FloatImgPixel *)calloc(sizeof(FloatImgPixel), iWidth*iHeight);
    if(piNew->pix == NULL) {
        free(piNew);
	return NULL;
    }
    
    return piNew;
}


void
Img_Delete(Img* pi) {
    if(pi != NULL) {
        if(pi->pix != NULL) free(pi->pix);
	free(pi);
    }
}


void
FloatImg_Delete(FloatImg* pi) {
    if(pi != NULL) {
        if(pi->pix != NULL) free(pi->pix);
	free(pi);
    }
}

int
Img_GetTk(Img* pi, char* szTkPhotoName) {
    Tk_PhotoHandle phImage;
    Tk_PhotoImageBlock pibImage;
    ImgPixel *pip;
    int iWidth, iHeight;
    int x, y;
    
    phImage = Tk_FindPhoto(szTkPhotoName);
    if(phImage == NULL) return 0;
    
    Tk_PhotoGetSize(phImage, &iWidth, &iHeight);
    Tk_PhotoGetImage(phImage, &pibImage);
    
    if(iWidth != pi->width || iHeight != pi->height) {
        if(pi->pix != NULL) free(pi->pix);
	pi->pix = (ImgPixel *)malloc(sizeof(ImgPixel)*iWidth*iHeight);
	if(pi->pix == NULL) return -1;
	pi->width = iWidth;
	pi->height = iHeight;
    }
    
    pip = pi->pix;
    for(y=0; y<iHeight; y++) {
        register unsigned char* pucScan =
	    &pibImage.pixelPtr[pibImage.pitch * y];
	
	for(x=0; x<iWidth*pibImage.pixelSize; x+=pibImage.pixelSize) {
	    pip->r = pucScan[x+pibImage.offset[0]];
	    pip->g = pucScan[x+pibImage.offset[1]];
	    pip->b = pucScan[x+pibImage.offset[2]];
	    pip->a = 255;
	    pip++;
	}
    }
    
    return 0;
}

int
Img_PutTk(const Img* pi, char* szTkPhotoName) {
    Tk_PhotoHandle phImage;
    Tk_PhotoImageBlock pibImage;
    int iWidth, iHeight;
    
    phImage = Tk_FindPhoto(szTkPhotoName);
    if(phImage == NULL) return -1;
    
    Tk_PhotoGetSize(phImage, &iWidth, &iHeight);
    Tk_PhotoGetImage(phImage, &pibImage);
    
    if(iWidth != pi->width || iHeight != pi->height)
        Tk_PhotoSetSize(phImage, pi->width, pi->height);
    
    /*
     * we're creating a Tk_PhotoImageBlock to describe
     * the Img structure.  Then, we can simply ask this
     * block to be copied into the widget itself.
     */
    pibImage.pixelPtr = (unsigned char *)pi->pix;
    pibImage.width = pi->width;
    pibImage.height = pi->height;
    pibImage.pitch = pi->width*sizeof(ImgPixel);
    pibImage.pixelSize = sizeof(ImgPixel);
    pibImage.offset[0] = Tk_Offset(ImgPixel, r);
    pibImage.offset[1] = Tk_Offset(ImgPixel, g);
    pibImage.offset[2] = Tk_Offset(ImgPixel, b);
    
    Tk_PhotoPutBlock(phImage, &pibImage, 0, 0, pi->width, pi->height);
    
    return 0;
}

void
Img_SetColor(const Img* image, int red, int green, int blue) {
    register int i, stop;
    ImgPixel color;
    
    color.r = red;
    color.g = green;
    color.b = blue;
    color.a = 255;
    
    for(i=0, stop=image->width*image->height; i<stop; i++)
	image->pix[i] = color;
}
